"""
OverlayPanel.py
Classes for panel and widget encapsulation for overlay screen
Author:      Del69
Created:     26/10/2008
"""
class OverlayPanel:
    """
    Class to hold a dynamic panel for the overlay screen
    """

    def __init__(self, panel_id, x_position = 0, y_position = 0):
        self.__id = panel_id
        self.__widgets = {}
        self.__added_to_screen = False
        self.__x_position = x_position
        self.__y_position = y_position
        self.__height = 0
        self.__width = 0
        self.__scrollable = False
        self.__panel_style = 0
        self.__title = ""
        self.__help_text = ""
        self.__vert_layout = True


    def set_position(self, x_position, y_position):
        """
        Sets the panels position on the screen
        """
        self.__x_position = x_position
        self.__y_position = y_position

    def set_minimum_size(self, min_height, min_width):
        """
        Sets the minimum size the panel can be
        """
        self.__minimum_height = min_height
        self.__mininum_width = min_width

    def get_widget(self, widgetID):
        """
        Gets a particular widget from this panel
        @Param widgetID: The string id of the widget
        @Return: widget class representing the widget or None on failure
        """
        if WidgetID in self.__Widgets:
            return self.__Widgets[WidgetID]
        else:
            return None

    def add_widget(self, widget):
        """
        Adds a widget to this panel
        @Param widget: widget class of the widget to add
        """
        self.__Widgets[widget.getID()] = widget

    def show(self, screen):
        """
        Shows this panel and its widgets.
        @param screen: The screen to show on
        @return: True on success, false on failure
        """
        if self.__added_to_screen:
            screen.show(self.__id)
            for widget in self.__widgets:
                widget_id = widget.get_id()
                screen.show(widget_id)
            return True
        else:
            return False

    def hide(self, screen):
        """
        Hides this panel and its widgets.
        @param screen: The screen to hide from
        @return: True on success, false on failure
        """
        if self.__added_to_screen:
            screen.hide(self.__id)
            for widget in self.__widgets:
                widget_id = widget.get_id()
                screen.hide(widget_id)
            return True
        else:
            return False

    def add_to_screen(self, screen):
        """
        Adds this panel to a screen
        @param screen: The screen to add to
        @return: True on success, False if already added
        """
        if not self.__added_to_screen:
            screen.addPanel(self.__id, self.__title, self.__help_text, self.__vert_layout, self.__scrollable,
                            self.__x_position, self.__y_position, self.__width, self.__height, self.__panel_style)
            self.__added_to_screen = True
            return True
        else:
            return False

    def add_widgets(self, screen):
        """
        Adds this panels widgets
        @param screen: The screen to add to
        """
        for widget in self.__widgets:
            widget.add_to_panel(self.__id, screen)

    def __determine_size(self):
        """
        Determines the size needed to encapsulate all the widgets
        """
        pass

    def update_controls(self):
        """
        Updates the widgets on the panel
        """
        pass

WIDGET_ARG_TYPES = {
    "label"  : ("AttachID", "Text", "UIFlags", "x", "y", "z", "FontType", "WidgetType", "Data1", "Data2"),
    "text"   : ("AttachID", "Text", "UIFlags", "x", "y", "z", "FontType", "WidgetType", "Data1", "Data2"),
}

class Widget:
    """
    Class that encapsulates a widget
    """
    def __init__(self, widget_type, widget_id, args):
        self.__type = widgetType
        self.__argNames = WIDGET_ARG_TYPES[widgetType]
        self.__data = args
        self.__id = widgetID

    def get_arg_names(self):
        """
        Gets the string names of the arguments of this widget
        @Return: Tuple containing the strings
        """
        return self.argNames

    def get_id(self):
        """
        Gets the widgets text ID
        @Return: widgets text ID string, none on no id
        """
        if self.__id != "":
            return self.__id
        else:
            return None

    def set_arg(self, argName, argValue):
        """
        Sets a particular argument
        @Param argName: String of the argument name
        @Param argValue: Value to set to the argument
        @Return: True on success, False on failure
        """
        if argName in self.__argNames:
            self.__data[argName] = argValue
            return True
        else:
            return False

    def get_arg(self, argName):
        """
        Gets a particular argument
        @Param argName: String of the argument name
        @Return: the value on success, None on failure
        """
        if argName in self.__argNames:
            return self.__data[argName]
        else:
            return None

    def get_size(self):
        """
        gets the size the widget occupies on the panel
        @return: tuple (x,y) size in pixels
        """
        pass

def main():
    """
    Entry point for testing execution
    """
    pass

if __name__ == '__main__':
    main()